"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Merger = void 0;
var translationFileModels_1 = require("./model/translationFileModels");
var strings_1 = require("@angular-devkit/core/src/utils/strings");
var stringUtils_1 = require("./stringUtils");
var FUZZY_THRESHOLD = 0.2;
function onlyXmlNodes(source) {
    return source.replace(/<[^>]+\/>|<([\w-]+)[^>]*>.*<\/\1>/g, '').trim() === '';
}
var Merger = /** @class */ (function () {
    function Merger(options, normalizedTranslationSourceFile, initialTranslationState) {
        this.options = options;
        this.normalizedTranslationSourceFile = normalizedTranslationSourceFile;
        this.initialTranslationState = initialTranslationState;
        this.idMapping = {};
    }
    Merger.prototype.mergeWithMapping = function (destFileContent, isSourceLang) {
        var _a, _b, _c;
        var inUnitsById = new Map(this.normalizedTranslationSourceFile.units.map(function (unit) { return [unit.id, unit]; }));
        var destUnitsById = new Map(destFileContent.units.map(function (unit) { return [unit.id, unit]; }));
        var allInUnitsWithoutDestinationUnit = this.normalizedTranslationSourceFile.units.filter(function (u) { return !destUnitsById.has(u.id); });
        var allInUnitsWithDestinationUnit = this.normalizedTranslationSourceFile.units.filter(function (u) { return destUnitsById.has(u.id); });
        // collect (potentially) obsolete units (defer actual removal to allow for fuzzy matching..):
        var removeNodes = destFileContent.units.filter(function (destUnit) { return !inUnitsById.has(destUnit.id); });
        var result = new translationFileModels_1.TranslationFile(__spreadArray([], destFileContent.units, true), destFileContent.sourceLang, destFileContent.targetLang, destFileContent.xmlHeader);
        for (var _i = 0, allInUnitsWithDestinationUnit_1 = allInUnitsWithDestinationUnit; _i < allInUnitsWithDestinationUnit_1.length; _i++) {
            var unit = allInUnitsWithDestinationUnit_1[_i];
            result.replaceUnit(unit, this.handle(unit, destUnitsById.get(unit.id), isSourceLang, removeNodes));
        }
        if ((_a = this.options.fuzzyMatch) !== null && _a !== void 0 ? _a : true) {
            var bestMatchesIdToUnits = new Map(allInUnitsWithoutDestinationUnit.map(function (inUnit) { return [inUnit.id, findCloseMatches(inUnit, removeNodes)]; }));
            var _loop_1 = function () {
                var inUnitId = (_b = getMinScoreId(bestMatchesIdToUnits)) !== null && _b !== void 0 ? _b : Array.from(bestMatchesIdToUnits.keys())[0];
                var bestMatch = (_c = bestMatchesIdToUnits.get(inUnitId)[0]) === null || _c === void 0 ? void 0 : _c.elem;
                var updated = this_1.handle(inUnitsById.get(inUnitId), bestMatch, isSourceLang, removeNodes);
                if (bestMatch) {
                    result.replaceUnit(bestMatch, updated);
                }
                else {
                    result.addUnit(updated);
                }
                bestMatchesIdToUnits.delete(inUnitId);
                if (bestMatch) {
                    bestMatchesIdToUnits.forEach(function (x) {
                        var i = x.findIndex(function (y) { return y.elem === bestMatch; });
                        if (i >= 0) {
                            x.splice(i, 1);
                        }
                    });
                }
            };
            var this_1 = this;
            while (bestMatchesIdToUnits.size) {
                _loop_1();
            }
        }
        else {
            for (var _d = 0, allInUnitsWithoutDestinationUnit_1 = allInUnitsWithoutDestinationUnit; _d < allInUnitsWithoutDestinationUnit_1.length; _d++) {
                var unit = allInUnitsWithoutDestinationUnit_1[_d];
                result.addUnit(this.handle(unit, undefined, isSourceLang, removeNodes));
            }
        }
        console.debug("removing ".concat(removeNodes.length, " ids: ").concat(removeNodes.map(function (n) { return n.id; }).join(', ')));
        return result.mapUnitsList(function (units) { return units.filter(function (unit) { return !removeNodes.includes(unit); }); });
    };
    Merger.prototype.normalize = function (source) {
        var _a, _b;
        if ((_a = this.options.collapseWhitespace) !== null && _a !== void 0 ? _a : true) {
            var adjusted = ((_b = this.options.prettyNestedTags) !== null && _b !== void 0 ? _b : true) && onlyXmlNodes(source) ? " ".concat(source.replace(/></g, '> <'), " ") : source;
            return (0, stringUtils_1.doCollapseWhitespace)(adjusted);
        }
        else {
            return source;
        }
    };
    /** Syncs `unit` to `destUnit` or adds `unit` as new, if `destUnit` is not given. */
    Merger.prototype.handle = function (unit, destUnit, isSourceLang, removeNodes) {
        var _a;
        if (destUnit) {
            var updatedDestUnit = destUnit;
            if (this.normalize(destUnit.source) !== this.normalize(unit.source)) {
                console.debug("update element with id \"".concat(unit.id, "\" with new source: ").concat(unit.source, " (was: ").concat(destUnit.source, ")"));
                var syncSourceLang = isSourceLang && destUnit.source === destUnit.target; // sync source language only if target is unchanged
                var syncTarget = syncSourceLang || isUntranslated(destUnit, this.initialTranslationState);
                updatedDestUnit = __assign(__assign({}, destUnit), { state: isSourceLang ? 'final' : this.initialTranslationState, source: unit.source, target: syncTarget && !(destUnit.target === undefined && this.options.newTranslationTargetsBlank === 'omit') ? unit.source : destUnit.target });
            }
            if (destUnit.id !== unit.id) {
                console.debug("matched unit with previous id \"".concat(destUnit.id, "\" to new id: \"").concat(unit.id, "\""));
                this.idMapping[destUnit.id] = unit.id;
                removeNodes.splice(removeNodes.indexOf(destUnit), 1);
                updatedDestUnit = __assign(__assign({}, updatedDestUnit), { id: unit.id });
            }
            updatedDestUnit = __assign(__assign({}, updatedDestUnit), { locations: unit.locations, meaning: unit.meaning, description: unit.description });
            return updatedDestUnit;
        }
        else {
            console.debug("adding element with id \"".concat(unit.id, "\""));
            return __assign(__assign({}, unit), { target: this.options.newTranslationTargetsBlank === 'omit' ? undefined : (((_a = this.options.newTranslationTargetsBlank) !== null && _a !== void 0 ? _a : false) && !isSourceLang ? '' : unit.source), state: isSourceLang ? 'final' : this.initialTranslationState });
        }
    };
    return Merger;
}());
exports.Merger = Merger;
function isUntranslated(destUnit, initialState) {
    return destUnit.state === initialState && (destUnit.target === undefined || destUnit.source === destUnit.target);
}
function findCloseMatches(originUnit, destUnits) {
    var originText = originUnit.source;
    return destUnits
        .map(function (n) { return ({
        elem: n,
        score: (0, strings_1.levenshtein)(originText, n.source) / originText.length
    }); })
        .filter(function (x) { return x.score < FUZZY_THRESHOLD; })
        .sort(function (a, b) { return a.score - b.score; });
}
function getMinScoreId(bestMatchesIdToUnits) {
    var minScoreId;
    var minScore = Number.MAX_VALUE;
    bestMatchesIdToUnits.forEach(function (x, id) {
        if (x.length) {
            var score = x[0].score;
            if (score < minScore) {
                minScore = score;
                minScoreId = id;
            }
        }
    });
    return minScoreId;
}
